"""
  Ratings.py
  
  Simple and efficient object rating system

  requirements

	allow numeric ratings, once per authenticated user.

	should support for future development

	data analysis of ratings to provide users with
	packages they will probably like.

	A brief chat on irc revealed two distinct approaches.

	A rating storage container with ratings stored in a
	OIbtree with indexes of object physical path and userid.

	or a hashed user index to numeric value store in a IIBTree
	with the rating object stored local to the rated object.
	 
  Author: Kapil Thangavelu <kvthan@wm.edu>
  Copyright: @2001 kapil thangavelu
  LICENSE: GPL

"""

from BTrees.IIBTree import IIBTree
from BTrees.Length import Length
from Products.Gideon.Objects.Namespace import *
from OFS.SimpleItem import SimpleItem
from Acquisition import Implicit
from AccessControl import getSecurityManager
from ZTUtils.Zope import make_query

class Rating(SimpleItem, Implicit):

	id = 'Ratings'
	
	security = ClassSecurityInfo()

	security.declarePublic('__call__')# logical
	__call__ = DTMLFile('ui/RatingView', globals())
	
	index_html = None

	security.declarePublic('thank_you')
	thank_you  = DTMLFile('ui/RatingThankYou', globals())

	security.declarePublic('already_voted')
	already_voted = DTMLFile('ui/RatingAlreadyVoted', globals())
	
	# hmm.. what does this mean for consistency
	# ie what happens to other caches.
	_v_avg_rating = None

	def __init__(self, id='Ratings'):
		self.id = 'Ratings'
		self._data = IIBTree()
		self._length = Length(0)

	security.declarePublic('add_rating')
	def add_rating(self, value=None, REQUEST=None):
		""" """
		if value is None: return
		else:
			try: int(value)
			except: return 

		uname = getSecurityManager().getUser().getId()
		
		if uname == 'Anonymous User' and REQUEST is not None: 
			# should redirect with form stuff in tact
			return_url = ''.join([self.get_login_url,
									make_query(value=value,
								return_url=self.absolute_url()+'/add_rating',)
					])
			
			REQUEST.RESPONSE.redirect(return_url)

		if self._data.has_key(hash(uname)):
			return self.already_voted(self)
		else:
			self._data[hash(uname)]=int(value)
			self._v_avg_rating = None
			self._length.change(1)
			#return self.thank_you(self)

		if REQUEST is not None:
			REQUEST.RESPONSE.redirect(self.aq_parent.absolute_url())

	security.declarePublic('get_number_votes')
	def get_number_votes(self):
		""" """
		return self._length()
			
	security.declarePublic('get_average_rating')
	def get_average_rating(self):
		
		if self._v_avg_rating is None:
			sum = 0
			length = 0
			if not self._data: return 0
			for v in self._data.values():
				sum += v
				length += 1
			self._v_avg_rating = sum/float(length)
			
		return self._v_avg_rating
			
	security.declarePublic('get_vote_url')
	def get_vote_url(self):
		return self.absolute_url()+'/add_rating'
